"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.values = values;
exports.default = void 0;

var _del = require("./immutable/del");

var _get = require("./immutable/get");

var _set = require("./immutable/set");

var _actions = require("./actions");

function values(state = {}, action) {
  switch (action && action.type) {
    case _actions.BATCH:
      {
        const {
          payload
        } = action;

        if (Array.isArray(payload)) {
          return payload.reduce((state, action) => values(state, action), state);
        }

        return state;
      }

    case _actions.ADD_VALUE:
      {
        const {
          payload: {
            index,
            path,
            value
          }
        } = action;
        const array = (0, _get.get)(state, path, []);

        if (Array.isArray(array)) {
          const i = isNaN(index) ? array.length : index;
          return (0, _set.set)(state, path, [...array.slice(0, i), value, ...array.slice(i)]);
        }

        return state;
      }

    case _actions.DEL_VALUE:
      {
        const {
          payload: {
            index,
            path
          }
        } = action;

        if (typeof index === 'undefined') {
          return (0, _del.del)(state, path);
        }

        const array = (0, _get.get)(state, path, []);
        const i = parseInt(index);

        if (Array.isArray(array) && i >= -array.length && i < array.length) {
          return (0, _set.set)(state, path, [...array.slice(0, i), ...array.slice(i + 1 || array.length)]);
        }

        return state;
      }

    case _actions.MOVE_VALUE:
      {
        const {
          payload: {
            fromIndex,
            path,
            toIndex
          }
        } = action;
        const array = (0, _get.get)(state, path, []);

        if (Array.isArray(array)) {
          let from = parseInt(fromIndex);
          let to = parseInt(toIndex);

          if (from < 0) {
            from += array.length;
          }

          if (to < 0) {
            to += array.length;
          }

          if (from !== to && from >= 0 && from < array.length && to >= 0 && to < array.length) {
            return (0, _set.set)(state, path, from < to ? [...array.slice(0, from), ...array.slice(from + 1, to + 1), array[from], ...array.slice(to + 1)] : [...array.slice(0, to), array[from], ...array.slice(to, from), ...array.slice(from + 1)]);
          }
        }

        return state;
      }

    case _actions.SET_VALUE:
      {
        const {
          payload: {
            merge,
            path,
            value
          }
        } = action;

        if (path && path.length) {
          return (0, _set.set)(state, path, value, merge);
        }

        return [null, undefined].includes(value) ? state : value;
      }

    default:
      return state;
  }
}

var _default = {
  values
};
exports.default = _default;
//# sourceMappingURL=reducers.js.map